@@ -26,11 +26,11 @@ source directory.
 #include "temp.h"
 #include "cfg.h"
 
-#define NEW_PARSE 1
+#define NEW_PARSE !__MSDOS__
 
 #define MAX_VAR_NAME MAX_TOKEN
 
-
+#if !__MSDOS__
 extern void do_image(void);
 extern void do_other(void);
 extern void do_disk(void);
@@ -57,8 +57,10 @@ CONFIG cf_identify[] = {
   { cft_strg, "image",		id_image,	NULL,NULL },
   { cft_strg, "other",		id_other,	NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
+#endif /* !__MSDOS__ */
 
 CONFIG cf_options[] = {
+#if !__MSDOS__
   { cft_strg, "append",		NULL,		NULL,NULL },
 /*#ifdef LCF_ATARAID*/
 #if 0
@@ -94,7 +96,9 @@ CONFIG cf_options[] = {
   { cft_strg, "loader",		NULL,		NULL,NULL },
   { cft_flag, "lock",		NULL,		NULL,NULL },
   { cft_flag, "mandatory",	NULL,		NULL,NULL },
+#endif /* !__MSDOS__ */
   { cft_strg, "map",		NULL,		NULL,NULL },
+#if !__MSDOS__
   { cft_flag, "master-boot",	NULL,		NULL,NULL },
   { cft_strg, "menu-scheme",	NULL,		NULL,NULL },
   { cft_strg, "menu-title",	NULL,		NULL,NULL },
@@ -118,8 +122,10 @@ CONFIG cf_options[] = {
   { cft_strg, "verbose",	NULL,		NULL,NULL },
   { cft_strg, "vga",		NULL,		NULL,NULL },
   { cft_strg, "vmdefault",	NULL,		NULL,NULL },
+#endif /* !__MSDOS__ */
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
+#if !__MSDOS__
 CONFIG cf_all[] = {
   { cft_strg, "alias",		NULL,		NULL,NULL },
   { cft_flag, "bmp-retain",	NULL,		NULL,NULL },
@@ -210,7 +216,7 @@ CONFIG cf_bitmap[] = {
   { cft_strg, "bmp-table",	NULL,		NULL,NULL },
   { cft_strg, "bmp-timer",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
-
+#endif /* !__MSDOS__ */
 
 #if NEW_PARSE
 
@@ -231,9 +237,22 @@ static int back = 0; /* can go back by one char */
 
 int cfg_open(char *name)
 {
-    if (!strcmp(name,"-")) file = stdin;
-    else if (!(file = fopen(file_name = name,"r"))) pdie(name);
     line_num = 1;
+    if (!strcmp(name,"-")) file = stdin;
+
+#if __MSDOS__
+    else if (!strcasecmp(name,"none")) return -1;
+#endif /* __MSDOS__ */
+
+    else if (!(file = fopen(file_name = name,"r"))) {
+#if !__MSDOS__
+		die("Cannot open: %s", name);
+#else  /* __MSDOS__ */
+		fprintf(errstd,"No configuration file: %s\n", name);
+		return -1;
+#endif /* __MSDOS__ */
+    }
+
     return fileno(file);
 }
 
@@ -309,7 +328,9 @@ static int next(void)
 	*put++ = ch;
     }
     *put = 0;
+#if !__MSDOS__
     if (!(var = getenv(buffer))) cfg_error("unknown variable \"%s\"",buffer);
+#endif /* !__MSDOS__ */
     return next();
 }
 
@@ -384,7 +405,10 @@ static char *cfg_get_token(void)
 		*here = 0;
 		return stralloc(buf);
 	    }
-	    if (!(escaped = (ch == '\\'))) *here++ = ch;
+#if !__MSDOS__
+	    if (!(escaped = (ch == '\\')))
+#endif /* !__MSDOS__ */
+		*here++ = ch;
 	}
 	ch = next();
     }
@@ -579,6 +603,7 @@ char *cfg_get_strg(CONFIG *table,char *item)
 }
 
 
+#if !__MSDOS__
 /* open the password file, passw=1 forces a new file to be created */
 
 static char *pw_file_name;
@@ -658,3 +683,5 @@ void cfg_alpha_check(void)
     }
 }
 #endif
+
+#endif /* !__MSDOS__ */
