@@ -2,7 +2,7 @@
   common.h  -  Common data structures and functions.
 
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2004 John Coffman.
+Copyright 1999-2005 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -35,6 +35,7 @@ source directory.
 
 #endif
 
+
 /*
 ;*/typedef struct {		/*
 							block	0
@@ -57,7 +58,7 @@ source directory.
 							block	0
 ;*/    char name[MAX_IMAGE_NAME+1];	/* image name, NUL terminated 
 						id_name:	.blkb	MAX_IMAGE_NAME_asm+1
-;*/    unsigned short password_crc[MAX_PW_CRC*(sizeof(int)/sizeof(short))];  /* 4 password CRC-32 values
+;*/    unsigned short password_crc[MAX_PW_CRC*(sizeof(INT4)/sizeof(short))];  /* 4 password CRC-32 values
 						id_password_crc:.blkb	MAX_PW_CRC_asm*4
 ;*/    unsigned short rd_size[2]; /* RAM disk size in sectors, 0 if none
 						id_rd_size:	.blkb	4		;don't change the order !!!
@@ -368,7 +369,7 @@ extern int bios_boot, bios_map;
 extern int ireloc, force_fs, extended_pt, query;
 extern int colormax;
 extern DL_BIOS bios_passes_dl;
-extern int boot_dev_nr,raid_flags,do_md_install,ndisk,zflag,eflag;
+extern int boot_dev_nr,raid_index,raid_flags,do_md_install,ndisk,zflag,eflag;
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
 extern int curr_drv_map;
 extern unsigned int prt_map[PRTMAP_SIZE+1];
@@ -440,5 +441,10 @@ unsigned int crc32 (unsigned char *cp, int nsize, unsigned int polynomial);
 void show_link(char *name);
 /* show what a link resolves to */
 
+#if __MSDOS__
+#undef comma
+#define comma 0
+#define semi ";"
+#endif
 #endif
 #endif
