@@ -1,7 +1,7 @@
 /* geometry.c  -  Device and file geometry computation */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2004 John Coffman.
+Copyright 1999-2005 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -673,7 +673,10 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    if (max_partno[major] == 15)  goto MASK15;
 	    if (max_partno[major] == 7)   goto MASK7;
 	    
-	    die("Sorry, don't know how to handle device 0x%04x",device);
+	    if (MAJOR(device)>=120 && MAJOR(device)<=127)
+		die("Linux experimental device 0x04x needs to be defined.\n"
+		    "Check 'man lilo.conf' under 'disk=' and 'max-partitions='", device);
+	    else die("Sorry, don't know how to handle device 0x%04x",device);
     }
     if (get_all) dev_close(&dev);
     if (verbose>=5) printf("exit geo_query_dev\n");
@@ -826,7 +829,8 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     /* Find underlying device for MD RAID */
     if (MAJOR(device) == MD_MAJOR) {
         char mdxxx[16];
-	int md_fd, pass;
+	int md_fd;
+/*	int pass;	*/
 	struct md_version md_version_info;
 	md_array_info_t md_array_info;
 	md_disk_info_t md_disk_info;
@@ -854,6 +858,18 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	if (md_array_info.level != 1)
 	    die("Only RAID1 devices are supported for boot images");
 	raid_limit = md_array_info.raid_disks + md_array_info.spare_disks;
+
+	/* version 22.7 */
+#if 1
+	is_raid = (device==boot_dev_nr);
+	md_disk_info.number = raid_index;
+	if (ioctl(md_fd,GET_DISK_INFO,&md_disk_info) < 0)
+	    die("GET_DISK_INFO: %s", mdxxx);
+	device = MKDEV(md_disk_info.major, md_disk_info.minor);
+
+#else		/* prior to 22.7 */
+{
+int pass;
    	for (pass = 0; pass < raid_limit; pass++) {
 	    md_disk_info.number = pass;
 	    if (ioctl(md_fd,GET_DISK_INFO,&md_disk_info) < 0)
@@ -876,6 +892,9 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 		break;
 	    }
 	}
+}
+#endif	/* end of code prior to version 22.7 */
+
 	close(md_fd);
     }
 
@@ -1043,7 +1062,7 @@ int geo_open_boot(GEOMETRY *geo,char *name)
     if (!S_ISREG(st.st_mode) && !S_ISBLK(st.st_mode))
 	die("%s: neither a reg. file nor a block dev.",name);
     geo->dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
-#if 0
+#if 1
     if (MAJOR(geo->dev) == MAJOR_FD) geo->fd = 0;
     else if ((geo->fd = open(name,O_NOACCESS)) < 0)
 	    die("open %s: %s",name,strerror(errno));
