@@ -1,7 +1,7 @@
 /* lilo.c  -  LILO command-line parameter processing */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2004 John Coffman.
+Copyright 1999-2005 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -27,14 +27,18 @@ source directory.
 #include "config.h"
 #include "lilo.h"
 #include "common.h"
+#include "cfg.h"
+
+#if !__MSDOS__
 #include "raid.h"
 #include "boot.h"
 #include "device.h"
 #include "flags.i"
 #include "geometry.h"
+#endif /* !__MSDOS__ */
 #include "map.h"
+#if !__MSDOS__
 #include "bsect.h"
-#include "cfg.h"
 #include "identify.h"
 #include "partition.h"
 #include "probe.h"
@@ -42,12 +46,16 @@ source directory.
 #include "loader.h"
 #include "md-int.h"
 #include "edit.h"
+#endif /* !__MSDOS__ */
+
 
 char *config_file;		/* actual name of the config file */
 int config_read;		/* readable by other than root */
 FILE *errstd;
 
 
+#if !__MSDOS__
+
 static void show_other(int fd)
 {
     BOOT_SECTOR buf[SETUPSECS-1];
@@ -88,10 +96,11 @@ static void show_other(int fd)
 	prtmap += 4;
     }
 }
-
+#endif /* !__MSDOS__ */
 
 static void show_images(char *map_file)
 {
+#if !__MSDOS__
     DESCR_SECTORS descrs;
     BOOT_SECTOR boot;
     MENUTABLE menu;
@@ -99,17 +108,27 @@ static void show_images(char *map_file)
     GEOMETRY geo;
     SECTOR_ADDR addr[4];
     char buffer[SECTOR_SIZE];
+#else /* __MSDOS */
+static DESCR_SECTORS descrs;
+static char buffer[SECTOR_SIZE];
+#endif /*__MSDOS__ */
     char *name;
     int fd,image;
     int tsecs;
     int tlinear, tlba32;
     unsigned short flags;
 
+#if !__MSDOS__
     fd = geo_open(&geo,map_file,O_RDONLY);
+#else  /* __MSDOS__ */
+    if ((fd = open(map_file,O_RDONLY))<=0)
+	die("Cannot open map file: %s", map_file);
+#endif /* __MSDOS__ */
     if (read(fd,buffer,SECTOR_SIZE) != SECTOR_SIZE)
 	die("read cmdline %s: %s",map_file,strerror(errno));
     if (read(fd,(char*)&descrs,sizeof(descrs)) != sizeof(descrs))
 	die("read descrs %s: %s",map_file,strerror(errno));
+#if !__MSDOS__
     if (lseek(fd, SECTOR_SIZE, SEEK_CUR) <= 0)	/* skip zero sector */
 	die("lseek over zero sector %s: %s",map_file,strerror(errno));
     if (read(fd,(char*)&param2,sizeof(param2)) != sizeof(param2))
@@ -172,7 +191,9 @@ static void show_images(char *map_file)
 	    printf("  Boot prompt message is %d bytes\n",param2.msg_len);
 	else printf("  Bitmap file is %d paragraphs (%d bytes)\n",
 			param2.msg_len, 16*param2.msg_len);
-	if (*(unsigned short *) buffer != DC_MAGIC || !buffer[2])
+/* 22.6.2 begin */
+	if (*(unsigned short *) buffer != DC_MAGIC /* || !buffer[2] */)
+/* 22.6.2 end */
 	    printf("  No default boot command line\n");
 	else printf("  Default boot command line: \"%s\"\n",buffer+2);
 	if (verbose>=3) {
@@ -180,15 +201,28 @@ static void show_images(char *map_file)
 	}
 	printf("Images:\n");
     }
-    for (image = 0; image < MAX_IMAGES; image++)
+/* 22.7 begin */
+    else	/* verbose==0 */
+#endif /* !__MSDOS__ */
+    {
+	if (*(unsigned short *) buffer == DC_MAGIC)
+	    printf("Default boot command line: \"%s\"\n",buffer+2);
+    }
+/* 22.7 end */
+    for (image = 0; image < MAX_IMAGES; image++) {
 	if (*(name = descrs.d.descr[image].name)) {
+#if __MSDOS__
+	    printf("%s\n", name
+#else /* !__MSDOS__ */
 	    fprintf(stdout,"%s%-" S(MAX_IMAGE_NAME) "s %s%s",verbose > 0 ? "  " : "",name,
 	      image ? "" : "*",
 #ifdef LCF_VIRTUAL
 	      descrs.d.descr[image].flags & FLAG_VMDEFAULT ? "@" :
 #endif
 	      ""
+#endif /* !__MSDOS__ */
 	      );
+#if !__MSDOS__
 	    if (verbose >= 2) {
 	        if (descrs.d.descr[image].start.device & (LINEAR_FLAG|LBA32_FLAG)) {
 		   unsigned int sector;
@@ -298,21 +332,21 @@ static void show_images(char *map_file)
 		    else printf("    Image data not found\n");
 		}
 	    }
-	}
+#endif /*  !__MSDOS__ */
+	} /* if */
+    } /* for */
 #undef OTHER
     (void) close(fd);
-#if 0
-    checksum = INIT_CKS;
-    for (i = 0; i < sizeof(descrs)/sizeof(unsigned short); i++)
-	checksum ^= ((unsigned short *) &descrs)[i];
-    if (!checksum) exit(0);
-#else
+#if !__MSDOS__
     if (descrs.l.checksum ==
-    	  crc32(descrs.sector, sizeof(descrs.l.sector), CRC_POLY1) )  exit(0);
-#endif
+    	  crc32(descrs.sector, sizeof(descrs.l.sector), CRC_POLY1) )
+#endif /* !__MSDOS__ */
+    	  	exit(0);
+#if !__MSDOS__
     fflush(stdout);
     fprintf(errstd,"Checksum error\n");
     exit(1);
+#endif /* !__MSDOS__ */
 }
 
 
@@ -320,18 +354,25 @@ static void usage(char *name)
 {
     char *here;
 
+#if !__MSDOS__
     here = strrchr(name,'/');
+#else /* __MSDOS__ */
+    here = strrchr(name,'\\');
+#endif /* __MSDOS__ */
     if (here) name = here+1;
     fprintf(errstd,"usage: %s [ -C config_file ] -q [ -m map_file ] "
       "[ -v N | -v ... ]\n",name);
+#if !__MSDOS__
     fprintf(errstd,"%7s%s [ -C config_file ] [ -b boot_device ] [ -c ] "
       "[ -g | -l | -L ]\n","",name);
     fprintf(errstd,"%12s[ -F ] [ -i boot_loader ] [ -m map_file ] [ -d delay ]\n","");
     fprintf(errstd,"%12s[ -v N | -v ... ] [ -t ] [ -s save_file | -S save_file ]\n",
       "");
     fprintf(errstd,"%12s[ -p ][ -P fix | -P ignore ] [ -r root_dir ] [ -w | -w+ ]\n","");
+#endif /* !__MSDOS__ */
     fprintf(errstd,"%7s%s [ -C config_file ] [ -m map_file ] "
       "-R [ word ... ]\n","",name);
+#if !__MSDOS__
     fprintf(errstd,"%7s%s [ -C config_file ] -I name [ options ]\n","",name);
     fprintf(errstd,"%7s%s [ -C config_file ] [ -s save_file ] "
       "-u | -U [ boot_device ]\n","",name);
@@ -339,6 +380,7 @@ static void usage(char *name)
     fprintf(errstd,"%7s%s -M /dev/XXX [ mbr | ext ]\tinstall master boot record\n","",name);
     fprintf(errstd,"%7s%s -T help \t\t\tlist additional options\n", "", name);
     fprintf(errstd,"%7s%s -X\t\t\t\tinternal compile-time options\n", "", name);
+#endif /* !__MSDOS__ */
     fprintf(errstd,"%7s%s -V [ -v ]\t\t\tversion information\n\n","",name);
     exit(1);
 }
@@ -350,14 +392,18 @@ int main(int argc,char **argv)
     char *tell_param, *uninst_dev, *param, *act1, *act2, ch;
 static char *bitmap_file;
     int more,version,uninstall,validate,activate,instmbr,geom;
-    struct stat st;
-    int fd, temp, tell_early=0;
+    int fd, temp=0, tell_early=0;
     int raid_offset;
+#if !__MSDOS__
+    struct stat st;
+#endif /* !__MSDOS__ */
 
     errstd = stderr;
 #if VERSION_MINOR>=50
     if (sizeof(MENUTABLE)!=256) die("MENUTABLE is not 256 bytes (common.h)");
+#if !__MSDOS__
     cfg_alpha_check();
+#endif /* !__MSDOS__ */
 #endif
     config_file = DFL_CONFIG;
     act1 = act2 = tell_param = 
@@ -365,11 +411,17 @@ static char *bitmap_file;
     do_md_install = zflag =
 	    version = uninstall = validate = activate = instmbr = 0;
     verbose = -1;
+#if !__MSDOS__
     name = *argv;
+#else  /* __MSDOS__ */
+    name = "lilo";
+#endif /* __MSDOS__ */
     argc--;
-    
+
+#if !__MSDOS__    
     if (atexit( (void(*)(void)) sync)) die("atexit(sync)");
     if (atexit( (void(*)(void)) purge)) die("atexit(purge)");
+#endif /* !__MSDOS__ */
     
     cfg_init(cf_options);
     while (argc && **++argv == '-') {
@@ -391,6 +443,7 @@ static char *bitmap_file;
 fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 #endif
 	switch (ch) {
+#if !__MSDOS__
 	    case 'A':
 		activate = 1;
 		act1 = param;
@@ -409,9 +462,11 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		cfg_set(cf_options,"compact",NULL,NULL);
 		compact = 1;
 		break;
+#endif /* !__MSDOS */
 	    case 'C':
 		config_file = param;
 		break;
+#if !__MSDOS__
 	    case 'd':
 		cfg_set(cf_options,"delay",param,NULL);
 		break;
@@ -449,9 +504,11 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 	    case 'L':
 		geometric |= AD_LBA32;
 		break;
+#endif /* !__MSDOS__ */
 	    case 'm':
 		cfg_set(cf_options,"map",param,NULL);
 		break;
+#if !__MSDOS__
 	    case 'M':
 		instmbr = 1;
 		act1 = param;
@@ -479,12 +536,15 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		else
 		    cfg_set(cf_options,param,NULL,NULL);
 		break;
+#endif /* !__MSDOS__ */
 	    case 'q':
 		query = 1;
 		break;
+#if !__MSDOS__
 	    case 'r':
 		new_root = param;
 		break;
+#endif /* !__MSDOS__ */
 	    case 'R':
 	        if (*(param = (*argv)+2)) argc++;
 	        else if (argc) param = *++argv;
@@ -505,6 +565,7 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);		    
 #endif
 		break;
+#if !__MSDOS__
 	    case 's':
 		cfg_set(cf_options,"backup",param,NULL);
 		break;
@@ -529,6 +590,7 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		}
 		uninst_dev = param;
 		break;
+#endif /* !__MSDOS__ */
 	    case 'v':
 	        if ((*argv)[2]) param = (*argv)+2;
 	        else if (argc && argv[1][0]>='0' && argv[1][0]<='9') {
@@ -545,6 +607,7 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    case 'V':
 		version = 1;
 		break;
+#if !__MSDOS__
 	    case 'w':
 		cfg_set(cf_options,"nowarn",NULL,NULL);
 		nowarn = 1;
@@ -553,8 +616,9 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    case 'x':
 		cfg_set(cf_options,RAID_EXTRA_BOOT,param,NULL);
 		break;
+#endif /* !__MSDOS__ */
 	    case 'X':
-#if VERSION_MINOR>=50
+#if VERSION_MINOR>=50 && !__MSDOS__
 {
 		printf(
 		"-DBOOT_PARAMS_1=%d -DBOOT_PARAMS_2=%d  PATH_MAX=%d"
@@ -565,8 +629,6 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		  PATH_MAX
 		     );
 }
-#endif
-#if VERSION_MINOR>=50
 {
 #include "bitmap.h"
 	RGB pal[16];
@@ -590,7 +652,9 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	printf("\nSECTOR_ADDR6=%d\n", (int)sizeof(SECTOR_ADDR6));
 #endif
 
-#endif
+#endif /* VERSION_MINOR>=50 && !__MSDOS__ */
+
+#if !__MSDOS__
 {
   unsigned int j;
   if (verbose>=5) {
@@ -667,26 +731,38 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 			LINUX_VERSION_CODE>>16,
 			LINUX_VERSION_CODE>>8 & 255,
 			LINUX_VERSION_CODE & 255);
+#endif /* !__MSDOS__ */
 		printf("\nMAX_IMAGES = %d\t\tc=%d, s=%d, i=%d, "
 				"l=%d, ll=%d, f=%d, d=%d, ld=%d\n\n",
 			MAX_IMAGES, (int)sizeof(char),
 			(int)sizeof(short), (int)sizeof(int),
-			(int)sizeof(long), (int)sizeof(long long),
+			(int)sizeof(long), (int)sizeof(
+#if !__MSDOS__
+			long
+#endif /* !__MSDOS__ */
+ 			     long),
 			(int)sizeof(float), (int)sizeof(double),
 			(int)sizeof(long double)
 			);
+#if __MSDOS__ || 1
+		printf("\nIMAGE_DESCR = %d   DESCR_SECTORS = %d\n",
+			(int)sizeof(IMAGE_DESCR), (int)sizeof(DESCR_SECTORS));
+#endif /* __MSDOS__ */
 		exit(0);
+#if !__MSDOS__
 	    case 'z':
 		zflag++;	/* force zero of MBR 8-byte area */
 		break;
 	    case 'Z':
 		cfg_set(cf_options,"bios-passes-dl",param,NULL);
 		break;
+#endif /* !__MSDOS__ */
 	    default:
 		usage(name);
 	}
     }
     if (argc) usage(name);
+#if !__MSDOS__
     if (!new_root) new_root = getenv("ROOT");
     if (new_root && *new_root) {
 	pp_fd = fopen(PARTITIONS, "r");
@@ -698,6 +774,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
     if (atexit(temp_remove)) die("atexit() failed");
     if (version+activate+instmbr+(tell_param!=NULL) > 1) usage(name);
     if (activate) do_activate(act1, act2);
+#endif /* !__MSDOS__ */
     if (verbose > 0 || version) {
        printf("LILO version %d.%d%s%s", VERSION_MAJOR, VERSION_MINOR,
 	      VERSION_EDIT, test ? " (test mode)" : "");
@@ -706,16 +783,16 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	    return 0;
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
-	       "Development beyond version 21 Copyright (C) 1999-2004 John Coffman\n"
+	       "Development beyond version 21 Copyright (C) 1999-2005 John Coffman\n"
 	       );
         if (verbose>0) {
-#if VERSION_MINOR>=50
+#if VERSION_MINOR>=50 && !__MSDOS__
 #include <sys/utsname.h>
 	    struct utsname buf;
 #endif
             printf("Released %s%s and compiled at %s on %s%s\n",
 		VERSION_DATE, comma ? "," : "", __TIME__, __DATE__, semi);
-#if VERSION_MINOR>=50
+#if VERSION_MINOR>=50 && !__MSDOS__
 	    if (uname(&buf)==0) {
 		printf("Running %s kernel %s\n", buf.sysname, buf.release);
 	    }
@@ -724,27 +801,32 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
         printf("\n");
         if (version) return 0;
     }
+
+    if (verbose > 0) errstd = stdout;
+#if !__MSDOS__
     preload_types();
     if (geometric & (geometric-1))
 	die ("Only one of '-g', '-l', or '-L' may be specified");
 
-    if (verbose > 0) errstd = stdout;
     if (tell_param) tell_early = strcasecmp(tell_param, "chrul")
     				&& strcasecmp(tell_param, "ebda");
     if (eflag) do_bitmap_edit(bitmap_file);
     if (tell_param && tell_early) probe_tell(tell_param);
     if (instmbr) do_install_mbr(act1, act2);
-    
+#endif /* !__MSDOS__ */    
 
     fd = cfg_open(config_file);
-    more = cfg_parse(cf_options);
+    more = fd<0 ? 0 : cfg_parse(cf_options);
     
+#if !__MSDOS__
     temp = cfg_get_flag(cf_options,"nowarn");
+#endif /* !__MSDOS__ */
     if (nowarn < 0) nowarn = 0;
     else nowarn = temp;
 
     if (verbose>=6) printf("main: cfg_parse returns %d\n", more);
 
+#if !__MSDOS__
     if (tell_param && !tell_early) probe_tell(tell_param);
 
     if (!nowarn) {
@@ -800,6 +882,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
     if (uninstall)
 	bsect_uninstall(uninst_dev ? uninst_dev : cfg_get_strg(cf_options,
 	      "boot"),cfg_get_strg(cf_options,"backup"),validate);
+#endif /* !__MSDOS__ */
 
     if (reboot_arg) {
 	map_patch_first(cfg_get_strg(cf_options,"map") ? cfg_get_strg(
@@ -807,22 +890,24 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	exit(0);
     }
 
+#if !__MSDOS__
     if ( (param = cfg_get_strg(cf_options,"bios-passes-dl")) ) {
 	if (strchr("YyTt1", *param)) bios_passes_dl = DL_GOOD;
 	if (strchr("NnFf0", *param)) bios_passes_dl = DL_BAD;
     }
     if (bios_passes_dl == DL_NOT_SET) 	check_bios();	/* in probe.c */
 
-    if (!nowarn && compact && (linear || lba32))
+    if (!nowarn && compact && (linear || lba32) && verbose>=2)
 	fprintf(errstd,"Warning: COMPACT may conflict with %s on some "
 		"systems\n", lba32 ? "LBA32" : "LINEAR");
 
     geo_init(cfg_get_strg(cf_options,"disktab"));
+#endif /* !__MSDOS__ */
     if (query)
 	show_images(!cfg_get_strg(cf_options,"map") ? MAP_FILE :
 	      cfg_get_strg(cf_options,"map"));
 
-
+#if !__MSDOS__
 /*************************************************/
 /*  Doing a real install (may be test mode)      */
 /*************************************************/
@@ -880,7 +965,9 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	      "altered.\n");
 	}
 	if (verbose>=4) dump_serial_nos();
-
+#else  /* __MSDOS__ */
+	die("No option switches specified:  -q, -R, or -V");
+#endif /* __MSDOS__ */
 	
     return 0;
 }
