@@ -17,6 +17,12 @@ source directory.
 #ifndef LILO_H
 #define LILO_H
 
+#if !__MSDOS__
+#define INT4 int
+#else
+#define INT4 long
+#endif /* !__MSDOS__ */
+
 /* This is the stuff to check the configuration:
  */
 #if defined(LCF_READONLY) && defined(LCF_REWRITE_TABLE)
@@ -51,9 +57,9 @@ source directory.
 
 #endif
 
-#ifdef HAS_VERSION_H
+#if defined(HAS_VERSION_H) && !__MSDOS__
 #include <linux/version.h>
-#endif
+#endif /* !__MSDOS__ */
 
 #ifndef LINUX_VERSION_CODE
 #define LINUX_VERSION_CODE 0
@@ -160,7 +166,7 @@ source directory.
 #if MAX_DESCR_SECS > MAX_DESCR_SECTORS
 # error "Maximum DSECS=x exceeded."
 #endif
-#define MAX_IMAGES      ((int)((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(int)-1)/sizeof(IMAGE_DESCR)))
+#define MAX_IMAGES      ((int)((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(INT4)-1)/sizeof(IMAGE_DESCR)))
 			  /* maximum number of images */
 
 #define COMMAND_LINE_SIZE	256	/* CL_LENGTH */
