@@ -21,10 +21,12 @@ source directory.
 
 #include "lilo.h"
 #include "common.h"
+#if !__MSDOS__
 #include "geometry.h"
+#endif /* !__MSDOS__ */
 #include "map.h"
 
-
+#if !__MSDOS__
 typedef struct _map_entry {
     SECTOR_ADDR addr;
     struct _map_entry *next;
@@ -32,10 +34,9 @@ typedef struct _map_entry {
 
 
 static MAP_ENTRY *map,*last;
-static GEOMETRY map_geo;
 static SECTOR_ADDR zero_addr;
 static int map_file;
-
+#endif /* !__MSDOS__ */
 
 void map_patch_first(char *name,char *str)
 {
@@ -47,7 +48,7 @@ void map_patch_first(char *name,char *str)
     if (strlen(str) >= SECTOR_SIZE-2)
 	die("map_patch_first: String is too long");
     if ((fd = open(name,O_RDWR)) < 0) die("open %s: %s",name,strerror(errno));
-    if (lseek(fd,SECTOR_SIZE,SEEK_SET) < 0) die("lseek %s: %s",name,strerror(errno));
+    if (lseek(fd,(off_t)SECTOR_SIZE,SEEK_SET) < 0) die("lseek %s: %s",name,strerror(errno));
     if (read(fd,(char *) &descrs,sizeof(descrs)) != sizeof(descrs))
 	die("read %s: %s",name,strerror(errno));
     for (start = str; *start && *start == ' '; start++);
@@ -64,7 +65,7 @@ void map_patch_first(char *name,char *str)
 	if (image == MAX_IMAGES) die("No image \"%s\" is defined",start);
 	if (end) *end = ' ';
     }
-    if (lseek(fd,0L,SEEK_SET) < 0) die("lseek %s: %s",name,strerror(errno));
+    if (lseek(fd,(off_t)0,SEEK_SET) < 0) die("lseek %s: %s",name,strerror(errno));
     magic = *str ? DC_MAGIC : 0;
     if ((size = write(fd,(char *) &magic,2)) < 0)
 	die("write %s: %s",name,strerror(errno));
@@ -75,6 +76,9 @@ void map_patch_first(char *name,char *str)
     if (close(fd) < 0) die("close %s: %s",name,strerror(errno));
 }
 
+#if !__MSDOS__
+
+static GEOMETRY map_geo;
 
 void map_create(char *name)
 {
@@ -472,4 +476,5 @@ off_t map_insert_data(unsigned char *data, int size)
     return here;
 }
 
+#endif /* !__MSDOS__ */
 
